static unsigned char abBuffer[300];
static unsigned char data[255];
static unsigned long StartTime;
static unsigned long ElapsedTime;

unsigned short multosSendAPDU(unsigned char bCLA, unsigned char bINS, unsigned char bP1, unsigned char bP2, unsigned short wLc, unsigned short wLe, unsigned short *La, unsigned char bCase, unsigned char *data, unsigned short maxWait)
{
  unsigned short wSW12;
  unsigned short wI;
  unsigned short wRxCount;
  unsigned short wSendLength;

  // abBuffer will contain the formatted message to send down the serial port
  wI = 0;
  abBuffer[0] = 0x80; // The "APDU Command" tag
  // Two byte space left for the length bytes - fill in later

  // Fill in the APDU header
  abBuffer[3] = bCLA;
  abBuffer[4] = bINS;
  abBuffer[5] = bP1;
  abBuffer[6] = bP2;
  wI = 7;

  if (wLc == 0) // Lc not set, Le could be set or be zero
  {
    // Case 1 commands don't have Lc or Le
    if(bCase != ISO_CASE_NO_DATA)
    {
      abBuffer[wI] = wLe / 256;
      abBuffer[wI + 1] = wLe % 256;
      wI += 2;
    }
  }
  else 
  {
    // Lc set, Le may be set
    abBuffer[wI] = wLc / 256;
    abBuffer[wI + 1] = wLc % 256;
    wI += 2; 

    // Copy in the APDU data
    memcpy(abBuffer + wI, data, wLc);
    wI += wLc;

    // Append Le if needed
    if (wLe > 0 || bCase == ISO_CASE_DATA_IN_OUT)
    {
      abBuffer[wI] = wLe / 256;
      abBuffer[wI + 1] = wLe % 256;
      wI += 2;
    }
  }

  // Set length of data
  wSendLength = wI;
  wI -= 3;
  abBuffer[1] = wI / 256;
  abBuffer[2] = wI % 256;
    
  // Send command - wait for it to get there
  for (wI = 0; wI < wSendLength; wI++)
  {
    gMultosSerial.write(abBuffer[wI]);   
  }

  // Read reply
  memset(abBuffer, 0, sizeof(abBuffer));

  // Wait for reply to start arriving
  StartTime = millis();
  ElapsedTime = 0;  
  while((gMultosSerial.available() == 0) && ((millis() - StartTime) < maxWait))
  {
    yield(); // Reset watchdog
  }

  // Read in all the bytes
  wI = 0;
  StartTime = millis();
  while( ((millis() - StartTime) < 500))
  {
    if (gMultosSerial.available() > 0)
    {
      abBuffer[wI] = gMultosSerial.read(); 
      wI++;
    }
    yield();
  }
  wRxCount = wI;

  // Return status always the last two bytes returned
  wSW12 = (abBuffer[wRxCount - 2] * 256) + abBuffer[wRxCount - 1];

  // Set up data to return
  *La = wRxCount - 5;  // serial protocol header + SW12 = 5 unsigned chars
  if(*La > 0)
    memcpy(data,abBuffer+3,*La);
    
  return (wSW12);
}

unsigned short SelectMF(void)
{
  unsigned char bCLA = 0x00; 
  unsigned char bINS = 0xA4; 
  unsigned char bP1 = 0x00; 
  unsigned char bP2 = 0x00; 
  unsigned short wLc = 0x02; 
  unsigned short wLe = 0x00; 
  unsigned short wLa; 
  unsigned char bCase = ISO_CASE_DATA_IN; 
  unsigned short wSW12;
  
  data[0] = 0x3F;
  data[1] = 0x00;
  
  wSW12 = multosSendAPDU (bCLA, bINS, bP1, bP2, wLc, wLe, &wLa, bCase, data, 1000);  
  return(wSW12);
}

unsigned short SelectAppByAID(void)
{
  unsigned char bCLA = 0x00; 
  unsigned char bINS = 0xA4; 
  unsigned char bP1 = 0x04; 
  unsigned char bP2 = 0x0C; 
  unsigned short wLc = 0x04; 
  unsigned short wLe  = 0x00; 
  unsigned short wLa; 
  unsigned char bCase = ISO_CASE_DATA_IN; 
  unsigned short wSW12;

  // AID of LED Flash application
  data[0] = 0xF4;
  data[1] = 0x00;
  data[2] = 0x00;
  data[3] = 0x01;

  wSW12 = multosSendAPDU(bCLA, bINS, bP1, bP2, wLc, wLe, &wLa, bCase, data, 1000); 
  return (wSW12);
}




