
/* eLoyalty application for SmartDeck */

#pragma attribute("aid", "f0 00 00 01")
#pragma attribute("name", "eloyalty")

#include <multos.h>

#define ERR_OK          0x9000
#define ERR_WRONGCLASS  0x6402
#define ERR_BAD_INS     0x6404
#define ERR_UNDERFLOW   0x6406

#define CMD_ADDPTS      0x10  /* Add points to the card */
#define CMD_SUBPTS      0x20  /* Deduct some points from the card */
#define CMD_QRYPTS      0x30  /* Return the number of points currently on the card */
#define CMD_APPINFO     0x40  /* Returns info about the application */
#define CMD_QRYID       0x44  /* Returns the ID info */
#define CMD_SETID       0x42  /* Sets the ID info */

#define MYAPP_CLA       0x70

/* APDU structure for QRYID and SETID commands. */
typedef struct
{
  char surname[20];
  char otherNames[40];
} APDU_ID;

/* APDU structure for ADDPTS, SUBPTRS, and QRYPTS commands. */
typedef struct
{
  int points;
} APDU_PTS;

/* APDU structure for APPINFO */
typedef struct
{
  char info[17];
} APDU_APPINFO;

static APDU_APPINFO app_info = { "Loyalty          " };

/* Data from APDU; this is placed at PB[0] */
#pragma melpublic
union
{
  APDU_ID id;
  APDU_PTS pts;
  APDU_APPINFO info;
} apdu_data;
#pragma melstatic

/* Static data for application. */
static APDU_ID id = { "Curtis", "Paul Lyndon" };
int points = 100;

void
main(void)
{
  int newPoints;

  /* Check class in APDU. */
  if (CLA != MYAPP_CLA)
    multosExitSW(ERR_WRONGCLASS);

  /* Decode instruction. */
  switch (INS)
    {
    case CMD_ADDPTS:
      /* Ensure case 3 command. */
      if (!multosCheckCase(3))
        multosExitSW(ERR_WRONGCLASS);

      /* Award points. */
      newPoints = points + apdu_data.pts.points;
      break;

    case CMD_SUBPTS:
      /* Ensure case 3 command. */
      if (!multosCheckCase(3))
        multosExitSW(ERR_WRONGCLASS);

      /* Deduct points. */
      newPoints = points - apdu_data.pts.points;
      break;

    case CMD_QRYPTS:
      /* Ensure case 2 command. */
      if (!multosCheckCase(2))
        multosExitSW(ERR_WRONGCLASS);

      /* Report points. */
      apdu_data.pts.points = points;
      multosExitLa(sizeof(APDU_PTS));
      break;

    case CMD_QRYID:
      /* Ensure case 2 command. */
      if (!multosCheckCase(2))
        multosExitSW(ERR_WRONGCLASS);

      /* Report ID. */
      apdu_data.id = id;

      /* Exit */
      multosExitLa(sizeof(APDU_ID));
      break;

    case CMD_SETID:
      /* Ensure case 3 command. */
      if (!multosCheckCase(3))
        multosExitSW(ERR_WRONGCLASS);
      id = apdu_data.id;

      /* Exit */
      multosExit();
      break;

    case CMD_APPINFO:
      /* Ensure case 2 command. */
      if (!multosCheckCase(2))
        multosExitSW(ERR_WRONGCLASS);
      apdu_data.info = app_info;

      /* Exit */
      multosExitLa(sizeof(APDU_APPINFO));
      break;

    default:
      multosExitSW(ERR_BAD_INS);
  }

  /* Common processing for point change. */
  if (newPoints < 0)
    multosExitSW(ERR_UNDERFLOW);
  points = newPoints;
}


