/*
 * hello_world.c
 *
 *  Created on: 12 Feb 2018
 *      Author: ctorr
 */

// Runs in EMBEDDED MODE:
// This application will output the message "Hello World" on the serial port when the
// Dev Board push button is pressed.

// It also includes commented out lines of code for making use of ULTRA_LOW power mode. Take care when using this
// as it applies to the whole chip and all embedded apps loaded to the chip will be effected.

// Application Identifier - used to select the application
#pragma attribute("aid","F4000003")
// The name that will appear in the Directory entry
#pragma attribute("name","Hello World")
// Permission to use peripherals (bit12)
#pragma attribute("access_list","1000")

// Standard MULTOS include file
#include <multos.h>

// Include file for Trust Anchor specific functions
#include <mi.h>

// Include for printf
#include <stdio.h>

// Define GPIO pin that the switch is connected to
#define SWITCH_PIN	5

// Declare stored variables. These are saved in the flash memory of the chip.
#pragma melstatic
BYTE lastError;
char message[] = "Hello World";
char message2[] = "Hello again!";

// Exception handler
void
abendHandler(void)
{
	lastError = miAbendReason();
	miPerformRestart();
}

void
main(void)
{
	BYTE id,info;

	// Get event details
	id = miGetEmbeddedEventId();
	info = miGetEmbeddedEventInfo();

	// Initialisation is only required once at startup
	if(id == MI_EMBEDDED_EVENT_STARTUP)
	{
		// Trap exceptions (abends)
		miSetAbendFunction((WORD)abendHandler);

		// Allocate GPIO pin 5 (the one connected to the on-board switch)
		miGPIOAllocatePin(SWITCH_PIN);

		// Register interest in an event on GPIO port 0 (pins GPIO0 -> GPIO7)...
		miRegisterEmbeddedEvent(MI_EMBEDDED_EVENT_GPIO,0);

		// ... and monitor the switch
		miGPIOMonitorPin (SWITCH_PIN, MI_GPIO_EVENT_HIGH_TO_LOW);

		// Set up the default serial port @ 9600 baud with a 100 byte buffer
		miSerialIOInitialise(MI_PACKAGE_PIN_27_GPIO_PIN_NUMBER,MI_PACKAGE_PIN_26_GPIO_PIN_NUMBER,9600,MI_SERIAL_IO_8_BITS_NO_PARITY_1_STOP_BIT,100);

		// Go into ultra low power mode. Current consumption will be tiny until woken up by the GPIO event.
		// Note: In ULTRA_LOW mode, timers don't work so you must use another event type to wake up the chip.
		//miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_ULTRA_LOW);
	}

	// If there has been an event on GPIO port 0
	else if (id == MI_EMBEDDED_EVENT_GPIO && info == 0)
	{
		// Switch to normal power mode
		//miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_NORMAL);

		// See if the switch pin has been triggered
		if(miGPIOMonitoredPinEventOccurred(SWITCH_PIN))
		{
			// Send the Hello World message over the serial port.
			miSerialIOTransmitBytes((BYTE*)message,sizeof(message));

			// printf also works and sends data to the current serial port
			printf(message2);
		}

		// Because of switch bounce, many events can be generated when pushing the switch. To clear those
		// unwanted events, the event queue should be flushed as follows.
		miFlushEmbeddedEventWithInfo(id,info);

		// Go back into ultra low power mode
		//miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_ULTRA_LOW);
	}
	// etc: check for other events

	// On exit, control returns to MULTOS for it to continue processing events.
}
