// Test uses four registers
// 1: IN: 4 bytes
// 2: IN: 1 byte
// 3: OUT: 4 bytes
// 4: OUT: 1 byte

#include <Wire.h>


// Globals
volatile byte currentRegister;
volatile byte copyReg1;
volatile byte copyReg2;

// The registers of this device
byte reg1[4];
byte reg2;
byte reg3[4];
byte reg4;

// Handle incoming bytes
void handleReceive(int numBytes) {  
	int i = 0;

	// There will always be at least one byte, the register
	currentRegister = Wire.read();
 
	if(currentRegister == 1)
	{
		while(i < 4)
		{
			reg1[i] = Wire.read();    
			i++;
		}
		// Flag ready to copy
		copyReg1 = 1;
	}		
	else if (currentRegister == 2)
	{
		reg2 = Wire.read();
		copyReg2 = 1;
	}

	// Junk anything else that's left over
	while(Wire.available())
	Wire.read();
}

// Output requested data depending on currently selected register
void handleRequest()
{
  int i = 0;

  if(currentRegister == 3)
  {       
    // Send it back
    Wire.write(reg3,4);
  }
  else if (currentRegister == 4)
  {  
    Wire.write(reg4);
  }
}

void setup()
{
	copyReg1 = 0;
	copyReg2 = 0;
	memset(reg1,0xAA, 4);
	memset(reg3,0xBB, 4);

  Serial.begin(9600); // For debugging messages
  
	Wire.begin(0x53); // Join I2C bus
	Wire.onReceive(handleReceive); // Input event handler
	Wire.onRequest(handleRequest); // Request for data handler

  Serial.println("Ready");
}

void loop()
{
  if(copyReg1)
  {
    Serial.println("Reg 1 copy");
	  memcpy(reg3,reg1,4);
	  copyReg1 = 0;
  }

  if(copyReg2)
  {
    Serial.println("Reg 2 copy");
	  reg4 = reg2;
	  copyReg2 = 0;
  }
}


