// MULTOS running in command mode either through ISO7816 interface or Serial interface(pin 18 low))
// Demonstrates using high level I2C calls to talk to an Arduino slave
// To run this, select the application then send the APDU CLA=00 INS=00 P1=00 P2=00

#pragma attribute("aid", "F4000004")
#pragma attribute("access_list", "1003")
#pragma attribute("name","I2C Master")

#include <multos.h>
#include <mi.h>
#include <string.h>
#include <ISO7816.h>
#define ISO_CASE_NO_DATA	1
#define ISO_CASE_DATA_OUT	2
#define ISO_CASE_DATA_IN	3
#define ISO_CASE_DATA_IN_OUT 4

// I2C connections
#define SDA_PIN MI_PACKAGE_PIN_28_GPIO_PIN_NUMBER
#define SCL_PIN MI_PACKAGE_PIN_29_GPIO_PIN_NUMBER

// Define supported APDU Class and Instruction bytes
#define SUPPORTED_CLA	0x70
#define TEST_INS		0x00

#pragma melsession
BOOL initialised;

void abendFnc(void)
{
	miPerformRestart();
}

void
main(void)
{
	BYTE msg[4];  // To hold message from Arduino
	QWORD rand;   // Random challenge to send to Arduino
	BYTE bRecvd;  // Single byte received
	BYTE bSend;   // Single byte sent

	// Validate APDU
	if(!multosCheckCase(ISO_CASE_NO_DATA))
		multosExitSW(ISO7816_SW_WRONG_PARAMS);
	if(CLA != SUPPORTED_CLA)
		multosExitSW(ISO7816_SW_CLA_NOT_SUPPORTED);
	if(INS != TEST_INS)
		multosExitSW(ISO7816_SW_INS_NOT_SUPPORTED);

#ifndef NO_DEBUG
	miDebuggingEnable();
#endif

	if(!initialised)
	{
		miSetAbendFunction((WORD)abendFnc);

		// Set up I2C interface using high-level functions
		miI2CInitialise(SCL_PIN,SDA_PIN,TRUE);
		miI2CSetPeripheralAddress(0x53);
		initialised = TRUE;
	}

	// Initialise reply buffer
	bRecvd = 0xCC;
	memset(msg,0xCC,4);

	// Get a random number - the challenge
	rand = multosGetRandomNumber();

	// Send the first four bytes of it to Arduino register 1
	miI2CRegisterWriteBytes(1,(BYTE*)&rand.msdw,4);
	miDelay(10);

	// A byte from the second part to Arduino register 2
	bSend = rand.lsdw & 0xFF;
	miI2CRegisterWriteByte(2,bSend);

	// Give Arduino time to copy things over to registers 3 and 4 then read back
	miDelay(100);
	miI2CRegisterReadBytes(3,msg,4);
	bRecvd = miI2CRegisterReadByte(4);

	// Compare
	if(bSend == bRecvd && memcmp((BYTE*)&rand.msdw,msg,4) == 0)
		multosExitSW(0x9000); // Passed

	multosExitSW(0x9001); // Failed
}
