#include <SPI.h>

// Local variables
int state;
volatile int idx;

#define BUFF_SIZES 100
static byte sendBuff[BUFF_SIZES];
static byte recvBuff[BUFF_SIZES];

// SPI interrupt routine
// Receive a byte and send the next one in the output buffer
ISR (SPI_STC_vect)
{
  byte c = SPDR;  // grab byte from SPI Data Register
  recvBuff[idx] = c;
  idx++;
  SPDR = sendBuff[idx];
} 

void initSendData()
{
  byte b = 2; 
  for(int x = 0; x < BUFF_SIZES; x++ )
  {
    sendBuff[x] = b;
    b += 2;
  }  
}
void setup()
{
  // Initialise variables
  state = 0;
  idx = 0;

  // Set up serial port
  Serial.begin(9600);
  
  // Initialise data that will be sent to MULTOS
  initSendData();
  
  // Enable SPI as slave.
  pinMode(SCK, INPUT);
  pinMode(MOSI, INPUT);
  pinMode(MISO, OUTPUT);
  pinMode(SS, INPUT); 
  // http://www.gammon.com.au/forum/?id=10892
  SPCR |= _BV(SPE);
  SPDR = sendBuff[0]; // First byte needs to be ready to send
  SPI.attachInterrupt();
}

void loop()
{ 
  switch(state)
  {     
    case 0: // Waiting for slave select
      if(digitalRead(SS) == LOW)        
        state = 1;      
    break;
    
    case 1: // Read data whilst SS remains low
      if(digitalRead(SS) == HIGH)
      {
        state = 2;
        Serial.print(idx);
        Serial.println(" bytes transferred");
      }
    break;
    
    case 2: // Check the data from MULTOS goes up in 3s      
    
      byte b = 3;
      int error = -1;
      for(int i = 0; i < BUFF_SIZES; i++)
      {
        if(recvBuff[i] != b)
          error = i;
        b += 3;
        Serial.print(recvBuff[i],16);
        Serial.print(",");
      }
      Serial.println();
      if(error == -1)
        Serial.println("Received OK");
       else
       {
        Serial.print ("Error at ");
        Serial.println(error);
       }
      
      state = 0; // Go back to waiting state

      // Set up ready to do it again
      initSendData();
      idx = 0;
      SPCR |= _BV(SPE);
      SPDR = sendBuff[0];
      memset(recvBuff,0,sizeof(recvBuff));
      
    break;
  }
}


