/*
 * spi_master.c
 *
 *  Created on: 13 Feb 2018
 *      Author: ctorr
 */


#pragma attribute("aid", "F4000006")
#pragma attribute("access_list", "1003")
#pragma attribute("name", "SPI Master")

#include <multos.h>
#include <stdlib.h>
#include <stdio.h>
#include <mi.h>

#define BUF_SIZES 100

// Define GPIO pin that the switch is connected to
#define SWITCH_PIN	5

#pragma melstatic

// An array defining the speed and SPI mode of each slave. For details see
// http://www.multos.com/learning/reference/mug_content/#miSPIInitialise
WORD spi_config[] = { 0x002B  };   // 375Kbps, CPOL = 0, CPHA = 0

#pragma melsession
BYTE sendBuff[BUF_SIZES];
BYTE recvBuff[BUF_SIZES];

void abendFnc(void)
{
	BYTE code;

	// Get abend reason code
	code = miAbendReason();

	// Output abend details on serial port
	printf("> Abend code %x\n",code);

	miPerformRestart();
}

void
initSendData(void)
{
	WORD i;
	BYTE x;

	x = 3;
	for(i = 0; i < BUF_SIZES; i++)
	{
		sendBuff[i] = x;
		x += 3;
	}
}

void
main(void)
{
	WORD i;
	BYTE x;
	BYTE match;
	BYTE id,info;

	// Get event details
	id = miGetEmbeddedEventId();
	info = miGetEmbeddedEventInfo();

	if(id == MI_EMBEDDED_EVENT_STARTUP)
	{
		// Set up default serial port
		miSerialIOInitialise(MI_PACKAGE_PIN_27_GPIO_PIN_NUMBER,MI_PACKAGE_PIN_26_GPIO_PIN_NUMBER,9600,MI_SERIAL_IO_8_BITS_NO_PARITY_1_STOP_BIT,100);

		// Set up trigger
		miGPIOAllocatePin(SWITCH_PIN);
		miRegisterEmbeddedEvent(MI_EMBEDDED_EVENT_GPIO,0);
		miGPIOMonitorPin (SWITCH_PIN, MI_GPIO_EVENT_HIGH_TO_LOW);

		// Initialise data to send
		initSendData();

		miDelay(50);

		// Set up SPI interface in MASTER mode with one slave and pull-ups enabled
		miSPIInitialise(MI_SPI_MODE_MASTER, 1, spi_config, TRUE);

		return;
	}

	// If there has been an event on GPIO port 0
	else if (id == MI_EMBEDDED_EVENT_GPIO && info == 0)
	{
		// See if the switch pin has been triggered
		if(miGPIOMonitoredPinEventOccurred(SWITCH_PIN))
		{
			printf("Exchanging data...\n");

			// Do the SPI exchange, deselect at end of exchange
			miSPISetSlave(1);
			miSPITransmitReceive(sendBuff, recvBuff, BUF_SIZES, TRUE);

			// Check the incoming data values are spaced by two
			x = 2;
			match = 1;
			for(i = 0; i < BUF_SIZES && match; i++)
			{
				if(recvBuff[i] != x)
					match = 0;

				x += 2;
			}

			// Output test result
			if(match)
				printf("> MULTOS recvd OK\n");
			else
				printf("> MULTOS error at %d, %x\n",i-1, recvBuff[i-1]);
			printf("> %x,%x,%x,%x,%x\n",recvBuff[0],recvBuff[1],recvBuff[2],recvBuff[3],recvBuff[4]);

			// Reset to do it again
			initSendData();
			multosClear(sizeof(recvBuff),recvBuff);

			// Cancel events caused by switch bounce
			miFlushEmbeddedEventWithInfo(id,info);
		}
	}
}
