/*
 * blink_led.c
 *
 *  Created on: 12 Feb 2018
 *      Author: ctorr
 */


// Runs in EMBEDDED MODE:
// This application will flash the on-board LED at a period determined by a timer. It includes an example of how to use low power mode.

// Application Identifier - used to select the application
#pragma attribute("aid","F4000002")
// The name that will appear in the Directory entry
#pragma attribute("name","Timed Blink")
// Permission to use peripherals (bit12)
#pragma attribute("access_list","1000")

// Standard MULTOS include file
#include <multos.h>

// Include file for Trust Anchor specific functions
#include <mi.h>

// Define GPIO pin that the LED is connected to
#define LED_PIN	6

// Declare stored variables. These are saved in the flash memory of the chip.
#pragma melstatic
BYTE lastError;
BYTE timerId;

// Exception handler
void
abendHandler(void)
{
	lastError = miAbendReason();
	miPerformRestart();
}

void
main(void)
{
	BYTE id,info;

	// Get event details
	id = miGetEmbeddedEventId();
	info = miGetEmbeddedEventInfo();

	// Initialisation is only required once at startup
	if(id == MI_EMBEDDED_EVENT_STARTUP)
	{
		// Trap exceptions (abends)
		miSetAbendFunction((WORD)abendHandler);

		// Allocate GPIO pin 6 (the one connected to the on-board LED) and set it as an Output.
		miGPIOAllocatePin(LED_PIN);
		miGPIOEnableOutputPin(LED_PIN);
		miGPIOSetPin(LED_PIN,MI_GPIO_PIN_STATE_LOW);

		// Set up the timer for 500ms and flag interest in the event
		timerId = miTimerCreate(500,MI_TIMER_TIME_UNIT_MILLISECONDS);
		miRegisterEmbeddedEvent(MI_EMBEDDED_EVENT_TIMER,timerId);

		// Register interest in other events, perhaps create another timer for a different use etc.

		// Put the device into low power mode
		//miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_LOW);

		// Start the timer running
		miTimerStart(timerId);
	}

	// If the timer has triggered
	else if (id == MI_EMBEDDED_EVENT_TIMER && info == timerId)
	{
		// Put device back into normal power mode
		//miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_NORMAL);

		// Flash the LED on and off
		miGPIOSetPin(LED_PIN,MI_GPIO_PIN_STATE_HIGH);
		miDelay(200);
		miGPIOSetPin(LED_PIN,MI_GPIO_PIN_STATE_LOW);

		// Clear the event for this timer - prevents another application from receiving the event
		// If you want other applications to process this event, don't flush the event
		miFlushEmbeddedEventWithInfo(MI_EMBEDDED_EVENT_TIMER,info);

		// Put the device back into low power mode
		//miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_LOW);

		// Restart the timer
		miTimerStart(timerId);
	}
	// etc: check for other events

	// On exit, control returns to MULTOS for it to continue processing events.
}
