/*
 * two_serial_ports.c
 *
 *  Created on: 13 Feb 2018
 *      Author: ctorr
 */

// Runs in embedded mode
// In this example, two serial ports are used (the maximum allowed). Messages arriving
// on one serial port are transmitted on the other.

#pragma attribute("aid", "F4000008")
#pragma attribute("access_list", "1003")
#pragma attribute("name", "Two Serial Ports")

#include <multos.h>
#include <stdlib.h>
#include <stdio.h>
#include <mi.h>

#define MAIN_SERIAL_PORT 0
#define SECONDARY_SERIAL_PORT 1
#define ONE_SERIAL_BYTE_EVENT 4 // = 1 << 2
#define MAX_MESSAGE 16

#pragma melsession
BYTE message[MAX_MESSAGE];
BYTE timerId;

void abendFnc(void)
{
	BYTE code;

	// Get abend reason code
	code = miAbendReason();

	// Output abend details on main serial port
	miSerialIOSetActivePort(MAIN_SERIAL_PORT);
	printf("> Abend code %x\n",code);
}

void
main(void)
{
	BYTE id,info;
	BYTE i,nRead;

	// Get event details
	id = miGetEmbeddedEventId();
	info = miGetEmbeddedEventInfo();

	if(id == MI_EMBEDDED_EVENT_STARTUP)
	{
		// Set up main serial port - connected to the PC
		miSerialIOSetActivePort(MAIN_SERIAL_PORT);
		miSerialIOInitialise(MI_PACKAGE_PIN_27_GPIO_PIN_NUMBER,MI_PACKAGE_PIN_26_GPIO_PIN_NUMBER,9600,MI_SERIAL_IO_8_BITS_NO_PARITY_1_STOP_BIT,100);

		// Register abend handler
		miSetAbendFunction((WORD)abendFnc);

		// Setup secondary port - connected to Arduino. G1 is Tx and G0 is Rx. Any of the pins G0 to G7 can be used.
		miSerialIOSetActivePort(SECONDARY_SERIAL_PORT);
		miSerialIOInitialise(MI_PACKAGE_PIN_6_GPIO_PIN_NUMBER,MI_PACKAGE_PIN_7_GPIO_PIN_NUMBER,9600,MI_SERIAL_IO_8_BITS_NO_PARITY_1_STOP_BIT,100);

		// Set up a timer to poll the secondary serial port
		timerId = miTimerCreate(500, MI_TIMER_TIME_UNIT_MILLISECONDS);
		miRegisterEmbeddedEvent(MI_EMBEDDED_EVENT_TIMER,timerId);
		miTimerStart(timerId);

	}
	// If timer event has occurred
	else if (id == MI_EMBEDDED_EVENT_TIMER && info == timerId)
	{
		// Select the serial port to check
		miSerialIOSetActivePort(SECONDARY_SERIAL_PORT);

		// Read data into a buffer from the serial port. It may all come in one go
		// or arrive in sections. This code allows for it to arrive in segments but
		// to never overflow the allocated buffer
		i = 0;
		while(miSerialIOGetNumReceivedBytes() && i < MAX_MESSAGE)
		{
			nRead = miSerialIOGetReceivedBytes(message + i,MAX_MESSAGE-i);
			i += nRead;
		}

		// This data can then be transmitted out on the main serial port.
		if(i > 0)
		{
			miSerialIOSetActivePort(MAIN_SERIAL_PORT);
			miSerialIOTransmitBytes(message,i);
		}

		// Restart the timer
		miTimerStart(timerId);
	}

}
